package com.jqtx.windows.component.impl;

import com.alibaba.fastjson2.JSONObject;
import com.jqtx.infrastructure.util.LoanMobileAESUtil;
import com.jqtx.infrastructure.util.SecurityUtils;
import com.jqtx.windows.component.LoanPlanComponent;
import com.jqtx.windows.component.SyncBuildComponent;
import com.jqtx.windows.component.model.CreditModel;
import com.jqtx.windows.component.model.LoanModel;
import com.jqtx.windows.component.model.LoanPlanModel;
import com.jqtx.windows.component.tsmodel.AddCaseRequest;
import com.jqtx.windows.component.tsmodel.ContactRequest;
import com.jqtx.windows.component.tsmodel.SimpleAttachRequest;
import com.jqtx.windows.component.tsmodel.UrgentContactRequest;
import com.jqtx.windows.utils.DateUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Service
public class SyncBuildComponentImpl implements SyncBuildComponent {

    @Value("${tiansicase.bizChannel}")
    private Long bizChannel;
    @Autowired
    private LoanPlanComponent loanPlanComponent;
    @Autowired
    private LoanComponentImpl loanComponent;
    @Autowired
    private CreditComponentImpl creditComponent;
    @Override
    public AddCaseRequest buildAddCaseRequest(String planNo) {
        //
        LoanPlanModel planRemindModel = loanPlanComponent.getPlanNoByPlanNo(planNo);
        if (planRemindModel == null) {
            return null;
        }
        //借款信息
        LoanModel loanModel = loanComponent.getByLoanNo(planRemindModel.getLoanNo());
        //查询用户信息
        CreditModel creditModel = creditComponent.getByOrderNo(loanModel.getOrderNo());
        String bizType = "ZN";

        AddCaseRequest request = new AddCaseRequest();
        request.setBizChannel(bizChannel);
        /** 客户基础信息 */
        request.setClientName(SecurityUtils.decryptAES(creditModel.getClientName()));
        request.setClientIdCard(SecurityUtils.decryptAES(creditModel.getCertNo()));
        request.setClientMobile(LoanMobileAESUtil.decrypt(creditModel.getClientCell()));

        /** 案件基础信息 */
        request.setLoanBankCard("");/** 还款卡号 */
        request.setLoanName("小花钱包-浙农项目");
        //获取配置
        request.setProductTypeName("小花钱包-浙农项目");/** 产品类型名称*/
        request.setProductTypeCode("XIAO_ZN"); /** 产品类型 */

        request.setLoanNo(loanModel.getLoanNo()); /** 借款编号 */
        request.setLoanCaseNo(planNo); /** 委托方案件编号 */
        request.setLoanDate(DateUtils.toDate(loanModel.getRaiseDate())); /** 放款日期 */
        request.setLoanBackDate(DateUtils.toDate(planRemindModel.getPlanDate())); /** 还款日期 */
        request.setLoanTotalPeriod(Integer.parseInt(loanModel.getLoanTerm())); /** 总期数 */
        request.setLoanLeftPeriod(Integer.parseInt(loanModel.getLoanTerm()) - planRemindModel.getPlanIndex()); /** 剩余期数 */
        request.setLoanCurrentPeriod(planRemindModel.getPlanIndex()); /** 当前期数 */
        request.setLoanCurrentPeriodDays(30); /** 当期天数 */
        request.setLoanDelegateTimes(1); /** 委外手数 */
        BigDecimal amt = planRemindModel.getPlanAmt().add(planRemindModel.getPlanFa()).subtract(planRemindModel.getBackedBen()).
                subtract(planRemindModel.getBackedFu()).subtract(planRemindModel.getBackedXi()).subtract(planRemindModel.getBackedFa())
                .subtract(planRemindModel.getBackedBurden()).subtract(planRemindModel.getBackedSpeaccount());
        request.setLoanDelegateAmt(amt); /** 委案金额 */
        request.setLoanTotalAmt(loanModel.getRaiseAmt());/** 本次借款总金额 */

        /** 合计金额 */
        BigDecimal backedAmt = loanPlanComponent.countBackedAmtByLoanNo(loanModel.getLoanNo());
        request.setTotalShouldRepay(loanModel.getLoanAmt());/** 全部应还金额*/
        request.setTotalBackedRepay(backedAmt);/** 全部已还金额*/
        request.setTotalUnBackedRepay(loanModel.getLoanAmt().subtract(backedAmt));/** 全部未还金额*/
//        request.setCapitalTotalShouldRepay(loan.getLoanAmt());/** 全部应还本金*/
//        request.setCapitalTotalBackedRepay(loanCollect.getPeriodAmt());/** 全部已还本金*/
        request.setCapitalPeriodShouldRepay(planRemindModel.getPlanBen());/** 当期应还本金*/
        request.setCapitalPeriodBackedRepay(planRemindModel.getBackedBen());/** 当期已还本金*/
//        request.setInterestTotalShouldRepay(loan.getRateAmt());/** 全部应还利息*/
//        request.setInterestTotalBackedRepay(BigDecimal.ZERO);/** 全部已还利息*/
        request.setInterestPeriodShouldRepay(planRemindModel.getPlanXi());/** 当期应还利息*/
        request.setInterestPeriodBackedRepay(planRemindModel.getBackedXi());/** 当期已还利息*/
//        request.setFeeTotalShouldRepay(loan.getServiceAmt());/** 全部应还服务费*/
//        request.setFeeTotalBackedRepay(BigDecimal.ZERO);/** 全部已还服务费*/
        BigDecimal FuAmt = planRemindModel.getPlanFu().add(planRemindModel.getPlanBurden()).add(planRemindModel.getPlanSpeaccount());
        request.setFeePeriodShouldRepay(FuAmt);/** 当期应还服务费*/
        BigDecimal backFu = planRemindModel.getBackedFu().add(planRemindModel.getBackedBurden()).add(planRemindModel.getBackedSpeaccount());
        request.setFeePeriodBackedRepay(backFu);/** 当期已还服务费*/
//        request.setPunishTotalShouldRepay(BigDecimal.ZERO);/** 全部应还罚息*/
//        request.setPunishTotalBackedRepay(BigDecimal.ZERO);/** 全部已还罚息*/
        request.setPunishPeriodShouldRepay(planRemindModel.getPlanFa());/** 当期应还罚息*/
        request.setPunishPeriodBackedRepay(planRemindModel.getBackedFa());/** 当期已还罚息*/

        /** 通讯录 */
        List<ContactRequest> contactRequests = new ArrayList<>();
        ContactRequest contactRequest = new ContactRequest();
        contactRequest.setContactMobile(LoanMobileAESUtil.decrypt(creditModel.getClientCell()));
        contactRequest.setContactName(SecurityUtils.decryptAES(creditModel.getClientName()));
        contactRequest.setRelation("无");
        contactRequests.add(contactRequest);
        request.setContactRequests(contactRequests);

        /** 附件处理*/
        List<SimpleAttachRequest> attachRequests = new ArrayList<>();
        request.setAttachments(attachRequests);
        request.setMemo("小花钱包-浙农项目");
        request.setFundChannel("小花钱包-浙农项目");
        return request;
    }
}
