package com.jqtx.windows.utils.http;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.HttpClientUtils;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.net.URI;
import java.nio.charset.Charset;
import java.util.*;

/**
 * http工具类
 *
 * @author shaohai.zhao
 * @version : HttpUtil.java, v 0.1 2018-08-08 19:30 shaohai.zhao Exp $
 */
public class HttpUtil extends HttpClientPools {

    /**
     * logger
     */
    private static final Logger logger = LoggerFactory.getLogger(HttpUtil.class);

  /**
   * 发送 POST 请求（HTTP），K-V形式
   *
   * @param apiUrl API接口URL
   * @param params 参数map
   */
  public static String doPost(String apiUrl, Map<String, String> headers,
      Map<String, String> params) {
    CloseableHttpClient httpClient = HttpClients.createDefault();
    String httpStr = null;
    HttpPost httpPost = new HttpPost(apiUrl);
    if (!CollectionUtils.isEmpty(headers)) {
      for (String key : headers.keySet()) {
        httpPost.setHeader(key, headers.get(key));
      }
    }
    CloseableHttpResponse response = null;
    try {
      httpPost.setConfig(requestConfig);
      List<NameValuePair> pairList = new ArrayList<NameValuePair>(params.size());
      for (Map.Entry<String, String> entry : params.entrySet()) {
        NameValuePair pair = new BasicNameValuePair(entry.getKey(), entry.getValue());
        pairList.add(pair);
      }
      httpPost.setEntity(new UrlEncodedFormEntity(pairList, Charset.forName("UTF-8")));
      response = httpClient.execute(httpPost);
      HttpEntity entity = response.getEntity();
      httpStr = EntityUtils.toString(entity, "UTF-8");
    } catch (Exception e) {
      logger.error("http请求失败,失败信息"+e.getMessage());
    } finally {
      if (response != null) {
        try {
          EntityUtils.consume(response.getEntity());
        } catch (IOException e) {
          logger.error("http请求失败,失败信息"+e.getMessage());
        }
      }
    }
    return httpStr;
  }

  /**
   * 发送 POST 请求（HTTP），JSON形式
   *
   * @param json json对象
   */
  public static String doPost(String apiUrl, Map<String, String> headers, String json) {
    CloseableHttpClient httpClient = HttpClients.createDefault();
    String httpStr = null;
    HttpPost httpPost = new HttpPost(apiUrl);
    CloseableHttpResponse response = null;
    if (!CollectionUtils.isEmpty(headers)) {
      for (String key : headers.keySet()) {
        httpPost.setHeader(key, headers.get(key));
      }
    }
    try {
      httpPost.setConfig(requestConfig);
      StringEntity stringEntity = new StringEntity(json, "UTF-8");//解决中文乱码问题
      stringEntity.setContentEncoding("UTF-8");
      stringEntity.setContentType("application/json");
      httpPost.setEntity(stringEntity);
      response = httpClient.execute(httpPost);
      HttpEntity entity = response.getEntity();
      httpStr = EntityUtils.toString(entity, "UTF-8");
    } catch (Exception e) {
      logger.error("http请求失败,失败信息"+e.getMessage());
    } finally {
      if (response != null) {
        try {
          EntityUtils.consume(response.getEntity());
        } catch (IOException e) {
          logger.error("http请求失败,失败信息"+e.getMessage());
        }
      }
    }
    return httpStr;
  }


  /**
   * 发送 SSL POST 请求（HTTPS），K-V形式
   *
   * @param apiUrl API接口URL
   * @param params 参数map
   */
  public static String doPostSSL(String apiUrl, Map<String, String> params) {
    CloseableHttpClient httpClient = getHttpClient();
    HttpPost httpPost = new HttpPost(apiUrl);
    CloseableHttpResponse response = null;
    String httpStr = null;
    try {
      httpPost.setConfig(requestConfig);
      List<NameValuePair> pairList = new ArrayList<NameValuePair>(params.size());
      for (Map.Entry<String, String> entry : params.entrySet()) {
        NameValuePair pair = new BasicNameValuePair(entry.getKey(), entry.getValue());
        pairList.add(pair);
      }
      httpPost.setEntity(new UrlEncodedFormEntity(pairList, Charset.forName("utf-8")));
      response = httpClient.execute(httpPost);
      HttpEntity entity = response.getEntity();
      httpStr = EntityUtils.toString(entity, "utf-8");
    } catch (Exception e) {
      logger.error("请求通联服务异常,失败信息"+ e.getMessage());
    } finally {
      HttpClientUtils.closeQuietly(response);
    }
    return httpStr;
  }

    /**
     * 路由提交方法
     */
    public static String doPostAll(String apiUrl, Map<String, String> params) {
        if (apiUrl == null) {
            return null;
        }
        if (apiUrl.indexOf("https:") != -1) {
            return doPostSSL(apiUrl, params);
        }
        return doPost(apiUrl, null, params);

    }

    public static String doGet(String url, Map<String, String> paramMap) {
        CloseableHttpClient httpClient = getHttpClient();
        HttpGet httpGet = new HttpGet(url);
        CloseableHttpResponse response = null;
        String httpStr = null;
        try {
            httpGet.setConfig(requestConfig);

            List<NameValuePair> params = new ArrayList<NameValuePair>();
            if (!CollectionUtils.isEmpty(paramMap)) {
                Set<String> keySet = paramMap.keySet();
                for (Iterator<String> it = keySet.iterator(); it.hasNext(); ) {
                    String next = it.next();
                    params.add(new BasicNameValuePair(next, paramMap.get(next)));
                }
            }
            String str = EntityUtils.toString(new UrlEncodedFormEntity(params, "utf-8"));
            httpGet.setURI(new URI(url + "?" + str));
            response = httpClient.execute(httpGet);
            HttpEntity entity = response.getEntity();
            httpStr = EntityUtils.toString(entity, "utf-8");
        } catch (Exception e) {
            logger.error("请求服务异常,失败信息"+e.getMessage());
        } finally {
            HttpClientUtils.closeQuietly(response);
        }
        return httpStr;
    }

}
